@@ -1,7 +1,7 @@
 /*
  * input.c
  *
- * Copyright 2013-2015 AmatCoder
+ * Copyright 2013-2018 AmatCoder
  *
  * This file is part of Mednaffe.
  *
@@ -25,11 +25,17 @@
 #include "toggles.h"
 #include "log.h"
 
+#if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) || defined(__DragonFly__)
+  #undef G_OS_UNIX
+#endif
+
 #ifdef G_OS_UNIX
  #include "joystick_linux.h"
  #include <linux/joystick.h>
  #include <fcntl.h>
-#else
+#endif
+
+#ifdef G_OS_WIN32
  #include <windows.h>
  #include "joystick_win.h"
 
@@ -44,16 +50,21 @@
 
 void close_channels(guidata *gui)
 {
-#ifdef G_OS_UNIX
   int a;
   for (a=0;a<9;a++)
   {
+    if (gui->joy[a].id != NULL)
+    {
+      g_free(gui->joy[a].id);
+    }
+
+#ifdef G_OS_UNIX
     if (gui->joy[a].channel != NULL)
     {
       g_io_channel_unref(gui->joy[a].channel);
     }
-  }
 #endif
+  }
 
 #ifdef G_OS_WIN32
 int i;
@@ -61,6 +72,11 @@ for(i=0;i<9;i++)
 {
   if(SDL_JoystickGetAttached(gui->joy[i].sdljoy))
     SDL_JoystickClose(gui->joy[i].sdljoy);
+
+  if (gui->joy[i].name != NULL)
+  {
+    g_free(gui->joy[i].name);
+  }
 }
    SDL_Quit();
 #endif
@@ -110,16 +126,19 @@ gchar* modkeys(gchar *key, gchar *value)
   if (key[0] == '\0') return NULL;
 
   gchar **items;
-  gchar *value2 = NULL;
-  guint num,i;
+  guint num;
 
   items = g_strsplit(key, "+", 4);
   num = g_strv_length(items);
 
   if (num>1)
   {
+    gchar *value2;
+    guint i;
+
     for (i=(num-1);i>0;i--)
     {
+      value2 = NULL;
       if (items[i][0]=='a') value2 = g_strconcat("Alt+", value, NULL);
       else if (items[i][0]=='c') value2 = g_strconcat("Ctrl+", value, NULL);
       else if (items[i][0]=='s') value2 = g_strconcat("Shift+", value, NULL);
@@ -127,6 +146,7 @@ gchar* modkeys(gchar *key, gchar *value)
       g_free(value);
       value = value2;
     }
+
   }
   g_strfreev(items);
 
@@ -136,31 +156,38 @@ gchar* modkeys(gchar *key, gchar *value)
 gchar* sdl2gdk(gchar *key)
 {
   gchar **line;
-  gchar **item;
-  gchar *gdk_key;
-  guint sdl_value;
+  gchar *gdk_key = NULL;
+  guint sdl_value = 0;
 
-  if (key[0] == '\0') return NULL;
+  line = g_strsplit(key, " ", 0);
 
-  line = g_strsplit(key, " ", 2);
+  if (g_strcmp0(line[1], "0x0"))
+  {
+    g_strfreev(line);
+    return NULL;
+  }
 
-  if (!line) return NULL;
+  if (line[2] != NULL)
+    sdl_value = g_ascii_strtod(line[2], NULL);
 
-  item = g_strsplit(line[1], "~", 2);
-
-  if (item[1] != NULL)
-    sdl_value = g_ascii_strtod(item[0], NULL);
-  else
-    sdl_value = g_ascii_strtod(line[1], NULL);
-
-  g_strfreev(item);
   g_strfreev(line);
 
-  if (sdl_value < 324)
-    gdk_key = g_strdup(sdl_to_gdk[sdl_value]);
-  else
-    gdk_key = NULL;
+  if ((sdl_value > 0) && (sdl_value < 245))
+  {
+    guint keyval;
 
+    gdk_keymap_translate_keyboard_state(gdk_keymap_get_for_display(gdk_display_get_default()),
+                                        sdl_to_gdk[sdl_value],
+                                        0,
+                                        0,
+                                        &keyval,
+                                        NULL,
+                                        NULL,
+                                        NULL);
+
+    if (keyval > 0)
+      gdk_key = g_strdup(gdk_keyval_name(gdk_keyval_to_upper(keyval)));
+  }
   return gdk_key;
 }
 
@@ -206,18 +233,13 @@ gboolean joy_watch( GIOChannel *channel, GIOCondition cond, guidata *gui)
   gchar *command;
   gchar *on;
   gchar *joyc;
-  unsigned int b;
 
   if (e.type == JS_EVENT_BUTTON) {
-    b = e.number;
-    on = g_strdup_printf("Button %02i", b);
-    //printf("Event: %08x\n", b);
-    joyc = g_strdup_printf("joystick %016llx %08x", gui->joy[a].id, b);
+    on = g_strdup_printf("Button %u", e.number);
+    joyc = g_strdup_printf("joystick %s button_%u", gui->joy[a].id, e.number);
   }
   else
   if (e.type == JS_EVENT_AXIS) {
-    //printf("%i\n", e.value);
-
     if (e.number % 2)
     {
       if (e.value<0) on = g_strdup_printf("Axis %i Up", e.number);
@@ -229,10 +251,10 @@ gboolean joy_watch( GIOChannel *channel, GIOCondition cond, guidata *gui)
         else on = g_strdup_printf("Axis %i Right", e.number);
     }
 
-    b = (0x8000 | e.number);
-    if (e.value<0) b = (0x4000 | b);
-    //printf("Event: %08x\n", b);
-    joyc = g_strdup_printf("joystick %016llx %08x", gui->joy[a].id, b);
+    if (e.value<0)
+      joyc = g_strdup_printf("joystick %s abs_%u-", gui->joy[a].id, e.number);
+    else
+      joyc = g_strdup_printf("joystick %s abs_%u+", gui->joy[a].id, e.number);
   }
 
   if ((gui->treepath != NULL) && (!gui->inputedited))
@@ -351,77 +373,88 @@ void read_input(guidata *gui)
         {
           gchar *value = hash2joy(hashkey, 2);
           gchar *hash = hash2joy(hashkey, 1);
-          long long unsigned int ll = g_ascii_strtoull(value, NULL, 16);
+          gchar *pch = g_strrstr(value, "_");
 
-          g_free(value);
+          if (pch != NULL)
+            pch++;
 
           int a;
           for (a=0;a<9;a++)
           {
-            if (gui->joy[a].id == g_ascii_strtoull(hash, NULL, 16))
+            if (g_strcmp0(gui->joy[a].id, hash) == 0)
             {
-              if (ll>0x7FFF)
+              gchar* value2;
+              if (value[0] == 'a')
               {
+                int ll = g_ascii_strtoll(pch, NULL, 10);
                 if (ll % 2)
                 {
-                  if (ll<0xc000) value = g_strdup_printf("Axis %01llx Down", ll & 0xFFF);
-                  else value = g_strdup_printf("Axis %01llx Up", ll & 0xFFF);
+                  if (g_strrstr(value, "+")) value2 = g_strdup_printf("Axis %i Down", ll);
+                  else value2 = g_strdup_printf("Axis %i Up", ll);
                 }
                 else
                 {
-                  if (ll<0xc000) value = g_strdup_printf("Axis %01llx Right", ll & 0xFFF);
-                  else value = g_strdup_printf("Axis %01llx Left", ll & 0xFFF);
+                  if (g_strrstr(value, "+")) value2 = g_strdup_printf("Axis %i Right", ll);
+                  else value2 = g_strdup_printf("Axis %i Left", ll);
                 }
               }
-              else value = g_strdup_printf("Button %02lli", ll & 0xFF);
+              else
+                value2 = g_strdup_printf("Button %s", pch);
 
-            gtk_list_store_set(GTK_LIST_STORE(model), &iter, 2, value, 3, gui->joy[a].name, -1);
-            g_free(value);
+              gtk_list_store_set(GTK_LIST_STORE(model), &iter, 2, value2, 3, gui->joy[a].name, -1);
+              g_free(value2);
+            }
+            //else gtk_list_store_set(GTK_LIST_STORE(model), &iter, 2, "", 3 , hash, -1);
           }
-         // else gtk_list_store_set(GTK_LIST_STORE(model), &iter, 2, "", 3 , hash, -1);
+          g_free(value);
+          g_free(hash);
         }
-        g_free(hash);
-      }
-      #endif
+        #endif
 
         #ifdef G_OS_WIN32
         else if (hashkey[0]=='j')
         {
           gchar *value = hash2joy(hashkey, 2);
           gchar *hash = hash2joy(hashkey, 1);
-          long long unsigned int ll = g_ascii_strtoull(value, NULL, 16);
+          gchar *pch = g_strrstr(value, "_");
 
-          g_free(value);
+          if (pch != NULL)
+            pch++;
+
+          int ll = g_ascii_strtoll(pch, NULL, 10);
 
           int a;
           for (a=0;a<9;a++)
           {
-            if (gui->joy[a].id == g_ascii_strtoull(hash, NULL, 16))
+            if (g_strcmp0(gui->joy[a].id, hash) == 0)
             {
-              if (ll>0x7FFF)
+
+              gchar* value2;
+
+              if (value[0] == 'a')
               {
                 if (ll % 2)
                 {
-                  if (ll<0xc000) value = g_strdup_printf("Axis %01I64x Down", ll & 0xFFF);
-                  else value = g_strdup_printf("Axis %01I64x Up", ll & 0xFFF);
+                  if (g_strrstr(value, "+")) value2 = g_strdup_printf("Axis %i Down", ll);
+                  else value2 = g_strdup_printf("Axis %i Up", ll);
                 }
                 else
                 {
-                  if (ll<0xc000) value = g_strdup_printf("Axis %01I64x Right", ll & 0xFFF);
-                  else value = g_strdup_printf("Axis %01I64x Left", ll & 0xFFF);
+                  if (g_strrstr(value, "+")) value2 = g_strdup_printf("Axis %i Right", ll);
+                  else value2 = g_strdup_printf("Axis %i Left", ll);
                 }
 
                 if (gui->joy[a].xinput==TRUE)
                 {
-                  if ((ll & 0xFFF)==4)
+                  if (ll == 4)
                   {
-                    g_free(value);
-                    value = g_strdup("Trigger Left");
+                    g_free(value2);
+                    value2 = g_strdup("Trigger Left");
                   }
-                  else if ((ll & 0xFFF)==5)
+                  else if (ll == 5)
                   {
-                    g_free(value);
-                    value = g_strdup("Trigger Right");
+                    g_free(value2);
+                    value2 = g_strdup("Trigger Right");
                   }
                 }
 
@@ -430,17 +463,18 @@ void read_input(guidata *gui)
               {
                 if (gui->joy[a].xinput==TRUE)
                 {
-                  value = xinput_map((ll & 0xFF));
+                  value2 = xinput_map(ll);
                 }
-                else value = g_strdup_printf("Button %02I64i", ll & 0xFF);
+                else value2 = g_strdup_printf("Button %s", pch);
               }
 
-             if ((value) && (gui->joy[a].name))
-               gtk_list_store_set(GTK_LIST_STORE(model), &iter, 2, value, 3, gui->joy[a].name, -1);
-            g_free(value);
+             if ((value2) && (gui->joy[a].name))
+               gtk_list_store_set(GTK_LIST_STORE(model), &iter, 2, value2, 3, gui->joy[a].name, -1);
+            g_free(value2);
           }
          // else gtk_list_store_set(GTK_LIST_STORE(model), &iter, 2, "", 3 , hash, -1);
         }
+        g_free(value);
         g_free(hash);
       }
       #endif
@@ -512,8 +546,6 @@ DWORD WINAPI joy_thread(LPVOID lpParam)
 
   if (bool==1) break;
 
-  //printf("Unhandled Event: %i\n", event.type);
-
   if ((event.type == SDL_JOYBUTTONDOWN) ||
       (event.type == SDL_JOYHATMOTION) ||
       ((event.type == SDL_JOYAXISMOTION) &&
@@ -535,8 +567,7 @@ DWORD WINAPI joy_thread(LPVOID lpParam)
       if ((gui->joy[a].xinput==TRUE) && (b>9))
         b=b+2;
 
-      joyc = g_strdup_printf("joystick %016I64x %08x", gui->joy[a].id, b);
-      //printf("Button Event: %s\n", joyc);
+      joyc = g_strdup_printf("joystick %s button_%u", gui->joy[a].id, b);
       which = a;
     }
     if (gui->joy[a].xinput==TRUE)
@@ -550,33 +581,34 @@ DWORD WINAPI joy_thread(LPVOID lpParam)
         a = GetControllerIndex(event.jhat.which, gui);
     if (a>-1)
     {
-    //printf("Value: %i\n", event.jhat.value);
-    int h=(((event.jhat.hat*2)-1) + SDL_JoystickNumAxes(gui->joy[a].sdljoy));
-    b = (0x8000 | h);
+    b = (((event.jhat.hat*2)-1) + SDL_JoystickNumAxes(gui->joy[a].sdljoy));
+
     if (event.jhat.value == SDL_HAT_UP)
     {
-      on = g_strdup_printf("Axis %i Up", h+2);
-      b = (0x4000 | b)+2;
+      on = g_strdup_printf("Axis %i Up", b+2);
+      b =b+2;
+      joyc = g_strdup_printf("joystick %s abs_%u-", gui->joy[a].id, b);
     }
     else if(event.jhat.value == SDL_HAT_DOWN)
     {
-      on = g_strdup_printf("Axis %i Down", h+2);
+      on = g_strdup_printf("Axis %i Down", b+2);
       b=b+2;
+      joyc = g_strdup_printf("joystick %s abs_%u+", gui->joy[a].id, b);
     }
     else if(event.jhat.value == SDL_HAT_RIGHT)
     {
-      on = g_strdup_printf("Axis %i Right", h+1);
+      on = g_strdup_printf("Axis %i Right", b+1);
       b=b+1;
+      joyc = g_strdup_printf("joystick %s abs_%u+", gui->joy[a].id, b);
     }
     else if(event.jhat.value == SDL_HAT_LEFT)
     {
-      on = g_strdup_printf("Axis %i Left", h+1);
-      b = (0x4000 | b)+1;
+      on = g_strdup_printf("Axis %i Left", b+1);
+      b = b+1;
+     joyc = g_strdup_printf("joystick %s abs_%u-", gui->joy[a].id, b);
     }
 
     which = a;
-    joyc = g_strdup_printf("joystick %016I64x %08x", gui->joy[a].id, b);
-    //printf("Hat Event: %s\n", joyc);
   }
 }
   else
@@ -585,53 +617,65 @@ DWORD WINAPI joy_thread(LPVOID lpParam)
     a = GetControllerIndex(event.jaxis.which, gui);
     if (a>-1)
     {
-    printf("Value: %i\n", event.jaxis.value);
-
+    char c;
     if (event.jaxis.axis % 2)
     {
-      if (gui->joy[a].xinput==TRUE)
+      if (event.jaxis.value<0)
       {
-        if (event.jaxis.value<0) on = g_strdup_printf("Axis %i Up", event.jaxis.axis);
-          else on = g_strdup_printf("Axis %i Down", event.jaxis.axis);
-
-          if (event.jaxis.axis==5)
-          {
-            g_free(on);
-            on = g_strdup("Trigger Right");
-          }
+        c = '-';
+        on = g_strdup_printf("Axis %i Up", event.jaxis.axis);
       }
       else
       {
-        if (event.jaxis.value<0) on = g_strdup_printf("Axis %i Up", event.jaxis.axis);
-         else on = g_strdup_printf("Axis %i Down", event.jaxis.axis);
+        c = '+';
+        on = g_strdup_printf("Axis %i Down", event.jaxis.axis);
+      }
+
+      if (gui->joy[a].xinput == TRUE)
+      {
+        if (event.jaxis.axis == 5)
+        {
+          g_free(on);
+          on = g_strdup("Trigger Right");
+        }
       }
     }
     else
     {
-      if (event.jaxis.value<0) {on = g_strdup_printf("Axis %i Left", event.jaxis.axis);}
-        else {on = g_strdup_printf("Axis %i Right", event.jaxis.axis);}
-
-      if (gui->joy[a].xinput==TRUE)
+      if (event.jaxis.value<0)
       {
-        if (event.jaxis.axis==4)
+        c = '-';
+        on = g_strdup_printf("Axis %i Left", event.jaxis.axis);
+      }
+      else
+      {
+        c = '+';
+        on = g_strdup_printf("Axis %i Right", event.jaxis.axis);
+      }
+
+      if (gui->joy[a].xinput == TRUE)
+      {
+        if (event.jaxis.axis == 4)
         {
+          c = '+';
           g_free(on);
           on = g_strdup("Trigger Left");
         }
       }
     }
     which = a;
-    b = (0x8000 | event.jaxis.axis);
-    if ((gui->joy[a].xinput==TRUE) && (event.jaxis.axis % 2))
+    b = event.jaxis.axis;
+    if (gui->joy[a].xinput == TRUE)
     {
-      if (event.jaxis.value>0) b = (0x4000 | b);
+      if (event.jaxis.axis % 2)
+      {
+        if (event.jaxis.value > 0)
+          c = '-';
+        else
+          c = '+';
+      }
     }
-    else
-    {
-      if (event.jaxis.value<0) b = (0x4000 | b);
-    }
-    joyc = g_strdup_printf("joystick %016I64x %08x", gui->joy[a].id, b);
-    printf("Axis Event: %s\n", joyc);
+    joyc = g_strdup_printf("joystick %s abs_%u%c", gui->joy[a].id, b, c);
     }
    }
   }
@@ -716,7 +760,7 @@ void on_input_clicked (GtkButton *button, guidata *gui)
   {
     if (GetJoy(a,gui)>0)
     {
-      gchar *id =g_strdup_printf(" - Unique ID: %016llx\n", gui->joy[a].id);
+      gchar *id =g_strdup_printf(" - Unique ID: %s\n", gui->joy[a].id);
       print_log("Joystick detected: ", FE, gui);
       print_log(gui->joy[a].name, FE, gui);
       print_log(id, FE, gui);
@@ -744,12 +788,11 @@ void on_input_clicked (GtkButton *button, guidata *gui)
   {
     gui->joy[i].sdljoy = SDL_JoystickOpen(i);
     gui->joy[i].xinput = FALSE;
+    gui->joy[i].id = NULL;
 
     if(!gui->joy[i].sdljoy)
     {
-      //printf("Couldn't open Joystick %i\n", i);
       gui->joy[i].sdl_id = 0;
-      gui->joy[i].id = 1;
       gui->joy[i].name = NULL;
     }
     else
@@ -761,36 +804,35 @@ void on_input_clicked (GtkButton *button, guidata *gui)
        char pszGUID[32];
        SDL_JoystickGetGUIDString(guid, (char *)pszGUID, 33);
 
-       /*printf("GUID: %s\n", pszGUID);*/
+       /*printf("GUID: %s\n", pszGUID);
 
-       /*if ((guid.data[0]=='x') &&
+         if ((guid.data[0]=='x') &&
           (guid.data[1]=='i') &&
           (guid.data[2]=='n') &&
           (guid.data[3]=='p') &&
           (guid.data[4]=='u') &&
           (guid.data[5]=='t'))*/
+
        if (g_strcmp0(pszGUID, "00000000000000000000000000000000")==0)
        {
-          int type, subtype;
+         int type =  SDL_JoystickDevType(gui->joy[i].sdljoy);      // Those functions do not exist in SDL
+         int subtype = SDL_JoystickDevSubType(gui->joy[i].sdljoy); // I patched it to expose XINPUT_CAPABILITIES
+         int flags =  SDL_JoystickCapsFlags(gui->joy[i].sdljoy);
+         int wButtons = SDL_JoystickCapsGamepadwButtons(gui->joy[i].sdljoy);
 
-          type =  SDL_JoystickDevType(gui->joy[i].sdljoy);      // Those functions do not exist in SDL
-          subtype = SDL_JoystickDevSubType(gui->joy[i].sdljoy); // I patched it to expose XINPUT_CAPABILITIES
+         gui->joy[i].id = g_strdup_printf("0x%016x%02x%02x%04x%04x%04x", 0, type, subtype, flags, wButtons, 0);
 
-          if ((type > -1) &&
-              (type < 2) &&
-              (subtype > -1 ) &&
-              (subtype < 20))
-          {
-            gui->joy[i].id = ((type << 24) | (subtype << 16));
-          }
-          else gui->joy[i].id = (0x00000000 << 24) | (0x00000001 << 16);
-
-          gui->joy[i].xinput = TRUE;
-          CheckDuplicatesXInput(i, gui);
+         gui->joy[i].xinput = TRUE;
+         CheckDuplicatesXInput(i, gui);
        }
        else
        {
-          GetJoy(i, gui);
+         int p1 = SDL_JoystickGuidProduct1(gui->joy[i].sdljoy);
+         int p2 = SDL_JoystickGuidProduct2(gui->joy[i].sdljoy);
+         int p3 = SDL_JoystickGuidProduct3(gui->joy[i].sdljoy);
+         char *p4 = SDL_JoystickGuidProduct4(gui->joy[i].sdljoy);
+
+         gui->joy[i].id = g_strdup_printf("0x%08x%04x%04x%02x%02x%02x%02x%02x%02x%02x%02x",p1, p2, p3, p4[0],p4[1],p4[2],p4[3],p4[4],p4[5],p4[6],p4[7]);
        }
        CheckDuplicates(i, gui);
     }
@@ -800,7 +842,7 @@ void on_input_clicked (GtkButton *button, guidata *gui)
   {
     if (gui->joy[i].name != NULL)
     {
-      gchar *id =g_strdup_printf(" - Unique ID: %016I64x\n", gui->joy[i].id);
+      gchar *id =g_strdup_printf(" - Unique ID: %s\n", gui->joy[i].id);
       print_log("Joystick detected: ", FE, gui);
       print_log(SDL_JoystickName(gui->joy[i].sdljoy), FE, gui);
       print_log(id, FE, gui);
@@ -808,36 +850,6 @@ void on_input_clicked (GtkButton *button, guidata *gui)
     }
   }
 
-   /* gchar *string;
-
-      gchar *dir = g_win32_get_package_installation_directory_of_module(NULL);
-      gchar *path =g_strconcat(dir, "\\stdout.txt", NULL);
-
-      if (g_file_get_contents(path, &string, NULL, NULL))
-      {
-        gchar **aline = g_strsplit(string, "\n", 0);
-        gint num = g_strv_length(aline);
-        gint l;
-        for(l=0;l<num-1;l++)
-        {
-          if (aline[l][2]=='J')
-          {
-             gint ii = g_ascii_digit_value(aline[l][11]);
-             printf ("\n%i\n",ii);
-             gchar *id = g_strrstr(aline[l],"ID:");
-             gui->joy[ii].id = g_strndup(id+=4,16);
-             //id = g_strrstr(aline[l],"Joystick ");
-             //gui->joy[ii].name = g_strndup(id+=13,(strlen(aline[l])-45));
-             printf (gui->joy[ii].id);
-             //printf (gui->joy[ii].name);
-          }
-        }
-        g_free(string);
-        g_strfreev(aline);
-      }
-      g_free(dir);
-      g_free(path); */
-
   #endif
 
   read_input(gui);
@@ -855,21 +867,6 @@ void on_input_clicked (GtkButton *button, guidata *gui)
   gtk_window_set_resizable(GTK_WINDOW(gui->inputwindow), FALSE);
 }
 
-guint gdk_to_sdl_keyval(guint gdk_key)
-{
-        if (!(gdk_key & 0xFF00))
-        {
-                gdk_key = g_ascii_tolower(gdk_key);
-                return gdk_key;
-        }
-
-        if (gdk_key & 0xFFFF0000) return 0;
-        if ((gdk_key & 0xFF) > 323) return 0;
-
-        guint sdl_key = gdk_to_sdl[gdk_key & 0xFF];
-        return sdl_key;
-}
-
 #ifdef G_OS_WIN32
 G_MODULE_EXPORT
 #endif
@@ -889,7 +886,6 @@ gboolean editable_key_cb(GtkWidget *ed, GdkEventKey *event, guidata *gui)
   gchar *fullcommand;
   gchar *key = NULL;
   gchar *key2;
-  guint nkey;
 
   if (gui->inputkeys)
   {
@@ -905,14 +901,14 @@ gboolean editable_key_cb(GtkWidget *ed, GdkEventKey *event, guidata *gui)
        ) return FALSE;
   }
 
-  nkey = gdk_to_sdl_keyval(event->keyval);
+  if (event->hardware_keycode > 244) return TRUE;
 
-  if (nkey > 323) return TRUE;
-
-  if (sdl_to_gdk[nkey] != NULL)
-    key = g_strdup(sdl_to_gdk[nkey]);//g_ascii_strup(gdk_keyval_name(event->keyval), -1);
+  if (gdk_to_sdl[event->hardware_keycode] != 0)
+    key = g_strdup(gdk_keyval_name(gdk_keyval_to_upper(event->keyval)));
   else return TRUE;
 
+  if (key == NULL) return TRUE;
+
   #ifdef G_OS_WIN32
     g_mutex_lock (&mutex);
     if (bool==1) {
@@ -960,7 +956,7 @@ gboolean editable_key_cb(GtkWidget *ed, GdkEventKey *event, guidata *gui)
 
   g_free(command);
   g_hash_table_replace(gui->clist, fullcommand, fullcommand);
-  key = g_strdup_printf("%i",nkey);
+  key = g_strdup_printf("%i", gdk_to_sdl[event->hardware_keycode]);
 
   if (gui->inputkeys)
   {
@@ -987,7 +983,7 @@ gboolean editable_key_cb(GtkWidget *ed, GdkEventKey *event, guidata *gui)
   }
 
   fullcommand++;
-  g_hash_table_insert(gui->hash, g_strdup(fullcommand), g_strconcat("keyboard ", key, NULL));
+  g_hash_table_insert(gui->hash, g_strdup(fullcommand), g_strconcat("keyboard 0x0 ", key, NULL));
   fullcommand--;
   g_free(key);
 
@@ -1019,7 +1015,6 @@ else
 #ifdef G_OS_WIN32
 void thread_watch(GPid pid, gint status, guidata *gui)
 {
-   //printf("Exiting from thread\n");
    g_mutex_lock (&mutex);
 
    if ((joyc != NULL) && (on != NULL) && (!gui->inputedited))
