@@ -211,7 +211,7 @@ void game_selected(GtkTreeSelection *treeselection, guidata *gui)
 
   if (gtk_tree_selection_get_selected(treeselection, &model, &iter))
   {
-    gchar *selected = NULL;
+    gchar *selected;
 
     g_free(gui->rom);
     g_free(gui->fullpath);
@@ -516,8 +516,7 @@ want to select the file manually?\n", &gui);
   #ifdef G_OS_WIN32
     home = g_path_get_dirname(gui.binpath);
   #else
-    home = g_getenv ("HOME");
-    if (!home) home = g_get_home_dir();
+    home = g_get_home_dir();
   #endif
     if (!home)
     {
@@ -526,25 +525,24 @@ want to select the file manually?\n", &gui);
     }
 
   #ifdef G_OS_WIN32
-    gchar *path = g_strconcat(home, "\\stdout.txt", NULL);
     gchar *qbin = g_strconcat("\"", gui.binpath, "\"", NULL);
-    gchar *cfg_path = g_strconcat(home, "\\mednafen-09x.cfg", NULL);
+    gchar *cfg_path = g_strconcat(home, "\\mednafen.cfg", NULL);
 
-    if ((g_file_get_contents(path, &stout, NULL, NULL)) &&
-        (g_file_test(cfg_path, G_FILE_TEST_IS_REGULAR)))
-    {}
-    else
+    if (!g_file_get_contents(cfg_path, &stout, NULL, NULL))
     {
     system(qbin);
       //Sleep(1000); /* race condition? */
-      g_file_get_contents(path, &stout, NULL, NULL);
+      g_file_get_contents(cfg_path, &stout, NULL, NULL);
     }
     g_free(cfg_path);
     g_free(qbin);
-    g_free(path);
 
   #else
-    g_spawn_command_line_sync(gui.binpath, &stout, NULL, NULL, NULL);
+    gchar *sterr = NULL;
+    gchar *command = g_shell_quote (gui.binpath);
+    g_spawn_command_line_sync(command, &stout, &sterr, NULL, NULL);
+    g_free(sterr);
+    g_free(command);
     //sleep (1); /* race condition? */
   #endif
 
@@ -560,7 +558,7 @@ want to select the file manually?\n", &gui);
   if (!check_version(stout, &gui))
    {
     show_error(
-    "Error: Mednafen version is not compatible.\nYou need 0.9.41 \
+    "Error: Mednafen version is not compatible.\nYou need 1.21.1 \
 version or above.\n");
     return 1; /* Items are not freed here */
   }
