@@ -315,7 +315,7 @@ void set_values(GtkBuilder *builder, guidata *gui)
       /*g_signal_connect(GTK_WIDGET(iterator->data), "focus-out-event",
        *                              G_CALLBACK(adj_focus_out), gui);*/
 #ifdef GTK3_ENABLED
-      if GTK_IS_SPIN_BUTTON(iterator->data)
+      if (GTK_IS_SPIN_BUTTON(iterator->data))
         gtk_widget_set_size_request(GTK_WIDGET(iterator->data), 150, -1);
 #endif
     }
@@ -354,28 +354,22 @@ gchar* get_cfg(const gchar *home, guidata *gui)
 
   /* Search for mednafen configuration file */
   #ifdef G_OS_WIN32
-    cfg_path = g_strconcat(home, "\\mednafen-09x.cfg", NULL);
+    cfg_path = g_strconcat(home, "\\mednafen.cfg", NULL);
   #else
-    cfg_path = g_strconcat(home, "/.mednafen/mednafen-09x.cfg", NULL);
+    /* Check if MEDNAFEN_HOME is set; if it is, then we use the path
+     * given as mednafen root directory. */
+    const gchar *mednafen_home =  g_getenv ("MEDNAFEN_HOME");
+    if (mednafen_home)
+        cfg_path = g_strconcat(mednafen_home, "/mednafen.cfg", NULL);
+    else
+        cfg_path = g_strconcat(home, "/.mednafen/mednafen.cfg", NULL);
   #endif
 
   if (g_file_test (cfg_path, G_FILE_TEST_IS_REGULAR))
-    print_log("Starting Mednaffe 0.8.6\n\
-Mednafen 09x configuration file found.\n", FE, gui);
+    print_log("Starting Mednaffe 0.8.8\n\
+Mednafen configuration file found.\n", FE, gui);
   else return NULL;
 
-  /*{
-  #ifdef G_OS_WIN32
-    cfg_path = g_strconcat(home, "\\mednafen.cfg", NULL);
-  #else
-    cfg_path = g_strconcat(home, "/.mednafen/mednafen.cfg", NULL);
-  #endif
-    if (g_file_test (cfg_path, G_FILE_TEST_IS_REGULAR))
-      printf(" Mednafen 08x configuration file found\n");
-    else
-      cfg_path = NULL;
-  }*/
-
   return cfg_path;
 }
 
@@ -387,41 +381,24 @@ gboolean check_version(gchar *stout, guidata *gui)
 
     gchar **achar = g_strsplit(stout, "\n", 0);
 
-    gchar *check =g_strrstr (achar[0], "0.9.");
+    gchar *check =g_strrstr (achar[0], " 1.");
     if (check == NULL) return FALSE;
 
-    gchar **aline = g_strsplit(check, " ", 2);
+    gchar **aline = g_strsplit(check, " ", 0);
 
-    if (strlen(aline[0])<6) return FALSE;
+    if (strlen(aline[1])<4) return FALSE;
 
-    if (aline[0][4]-'0'<4)
+    if (aline[1][2]-'0'<2)
       return FALSE;
 
-    /* if ((aline[1][13]=='3') && ((aline[1][14]-'0')<9)) 
-      return FALSE; */
-    /*
-    if (strlen(aline[1])>16)
-    {
-      if ((aline[1][13]=='3') && (aline[1][14]=='6') && ((aline[1][16]-'0')<2))
-        return FALSE;
-    }
-    else
-    {
-    if ((aline[1][13]=='3') && ((aline[1][14]-'0')<7))
-      return FALSE;
-    }
-  */
     print_log("Mednafen version detected: ", FE, gui);
-    print_log(aline[0], FE, gui);
-    #ifdef G_OS_UNIX
-      print_log("\n", FE, gui);
-    #endif
-    print_log("----\n", FE, gui);
+    print_log(aline[1], FE, gui);
+    print_log("\n----\n", FE, gui);
 
     GtkStatusbar *sbversion = GTK_STATUSBAR(gtk_builder_get_object(gui->builder, "sbversion"));
-    gtk_statusbar_push(GTK_STATUSBAR(sbversion), 1, aline[0]);
+    gtk_statusbar_push(GTK_STATUSBAR(sbversion), 1, aline[1]);
 
-    gchar *tooltip = g_strconcat("Mednafen version detected: ",aline[0], "\nPath: ", gui->binpath, NULL);    
+    gchar *tooltip = g_strconcat("Mednafen version detected: ",aline[1], "\nPath: ", gui->binpath, NULL);
     gtk_widget_set_tooltip_text(GTK_WIDGET(sbversion), tooltip);
     g_free(tooltip);
 
